#!/usr/bin/env python
# -*- coding:utf-8 -*- 


##############################################################################
## license :
##============================================================================
##
## File :        MetaExperiment.py
## 
## Project :     Metadata for high level experiment
##
## This file is part of Tango device class.
## 
## Tango is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## Tango is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with Tango.  If not, see <http://www.gnu.org/licenses/>.
## 
##
## $Author :      andy.gotz$
##
## $Revision :    $
##
## $Date :        $
##
## $HeadUrl :     $
##============================================================================
##            This file is generated by POGO
##    (Program Obviously used to Generate tango Object)
##############################################################################

__all__ = ["MetaExperiment", "MetaExperimentClass", "main"]

__docformat__ = 'restructuredtext'

import tango
import sys

from datetime import datetime
import logging
from metadata_manager.icat_ingest_metadata.investigation import investigation
from metadata_manager.MessagingClient import StompClient
from metadata_manager.TangoLoggingHandler import TangoLoggingHandler

try:
    import graypy
except ImportError:
    graypy = None

class MetaExperiment (tango.LatestDeviceImpl):
    """Metadata for high level experiment"""
    

    def __init__(self, cl, name):
        tango.LatestDeviceImpl.__init__(self,cl,name)
        self.debug_stream("In __init__()")
        MetaExperiment.init_device(self)
      
    def delete_device(self):
        self.debug_stream("In delete_device()")      
        self.client.disconnect()

    def init_device(self):
        self.debug_stream("In init_device()")
        self.get_device_properties(self.get_device_class())
        self.attr_proposal_read = ''
        self.attr_sample_read = ''
        self.attr_dataRoot_read = ''       
        self.attr_proposal_read = 'please enter'
        self.attr_sample_read = 'please enter'
        self.attr_dataRoot_read = '/data/visitor/'
        self.set_change_event("Proposal", True, False)
        self.set_change_event("Sample", True, False)
        self.set_change_event("DataRoot", True, False)
        self.set_state(tango.DevState.ON)

        # logger bridge
        self.l = logging.getLogger()
        self.l.setLevel(logging.DEBUG)
        self.l.addHandler(TangoLoggingHandler(self))
        if graypy != None:
            self.l.info("Init MetaExperiment logging. graylogServer=%s port=%s instrument=%s" % (self.graylogServer, self.graylogPort, self.beamlineID))               
            try:
                self.l.addHandler(graypy.GELFHandler(self.graylogServer, self.graylogPort))                
            except:
                try:
                    self.l.addHandler(graypy.GELFUDPHandler(self.graylogServer, self.graylogPort)) 
                except:
                    traceback.print_exc(file=sys.stdout)
         
        # JMS client
        self.client = StompClient(self.queueURLs, self.queueName, self.beamlineID)
        self.info_stream("Configuration URL=%s" %self.client.getConfigURL())
        self.Connect()

        # self client
        self.proxy = tango.DeviceProxy(self.get_name())
        # mimic write_hardware without side effects
        db = tango.Util.instance().get_database()
        attr_map = db.get_device_attribute_property(self.get_name(),['proposal','sample','dataRoot'])
        self.attr_proposal_read = attr_map['proposal'].get('__value',[''])[0]        
        self.attr_sample_read = attr_map['sample'].get('__value',[''])[0]
        self.attr_dataRoot_read = attr_map['dataRoot'].get('__value',[''])[0]
        if self.attr_proposal_read:        
            self.info_stream("Recovered proposal %s" %self.attr_proposal_read)
        if self.attr_sample_read:
            self.info_stream("Recovered sample name %s" %self.attr_sample_read)
        if self.attr_dataRoot_read:
            self.info_stream("Recovered data root %s" %self.attr_dataRoot_read)
        # we need to push them in this order
        self.push_change_event("Proposal", self.attr_proposal_read)
        self.push_change_event("Sample", self.attr_sample_read)
        self.push_change_event("DataRoot", self.attr_dataRoot_read)
        self.UpdateState()

        #----- PROTECTED REGION END -----#	//	MetaExperiment.init_device

    def always_executed_hook(self):
        self.debug_stream("In always_executed_hook()")
        #----- PROTECTED REGION ID(MetaExperiment.always_executed_hook) ENABLED START -----#
 
        #----- PROTECTED REGION END -----#	//	MetaExperiment.always_executed_hook

    #--------------------------------------------------------------------------
    #    MetaExperiment read/write attribute methods
    #--------------------------------------------------------------------------
    
    def read_proposal(self, attr):
        self.debug_stream("In read_proposal()")
        #----- PROTECTED REGION ID(MetaExperiment.proposal_read) ENABLED START -----#
        attr.set_value(self.attr_proposal_read)
        
        #----- PROTECTED REGION END -----#	//	MetaExperiment.proposal_read
        
    def write_proposal(self, attr):
        self.debug_stream("In write_proposal()")
        data = attr.get_write_value()
        #----- PROTECTED REGION ID(MetaExperiment.proposal_write) ENABLED START -----#
        # new proposal (investigation) registration, if not empty
        if data and (data != self.attr_proposal_read):
            self.CreateInvestigation(data)
        self.attr_proposal_read = data
        self.push_change_event("Proposal", self.attr_proposal_read)
        self.proxy.write_attribute("sample", "please enter")
        self.proxy.write_attribute("dataRoot", "/data/visitor/")
        self.UpdateState()
        #----- PROTECTED REGION END -----#	//	MetaExperiment.proposal_write
        
    def read_sample(self, attr):
        self.debug_stream("In read_sample()")
        #----- PROTECTED REGION ID(MetaExperiment.sample_read) ENABLED START -----#
        attr.set_value(self.attr_sample_read)
        
        #----- PROTECTED REGION END -----#	//	MetaExperiment.sample_read
        
    def write_sample(self, attr):
        self.debug_stream("In write_sample()")
        data = attr.get_write_value()
        #----- PROTECTED REGION ID(MetaExperiment.sample_write) ENABLED START -----#
        self.attr_sample_read = data
        self.push_change_event("Sample", self.attr_sample_read)
        self.UpdateState()
        #----- PROTECTED REGION END -----#	//	MetaExperiment.sample_write
        
    def read_dataRoot(self, attr):
        self.debug_stream("In read_dataRoot()")
        #----- PROTECTED REGION ID(MetaExperiment.dataRoot_read) ENABLED START -----#
        attr.set_value(self.attr_dataRoot_read)
        #----- PROTECTED REGION END -----#	//	MetaExperiment.dataRoot_read
        
    def write_dataRoot(self, attr):
        self.debug_stream("In write_dataRoot()")
        data = attr.get_write_value()
        #----- PROTECTED REGION ID(MetaExperiment.dataRoot_write) ENABLED START -----#
        self.attr_dataRoot_read = data
        self.push_change_event("DataRoot", self.attr_dataRoot_read)
        self.UpdateState()
        #----- PROTECTED REGION END -----#	//	MetaExperiment.dataRoot_write
        
    
    
            
    def read_attr_hardware(self, data):
        self.debug_stream("In read_attr_hardware()")
        #----- PROTECTED REGION ID(MetaExperiment.read_attr_hardware) ENABLED START -----#
        
        #----- PROTECTED REGION END -----#	//	MetaExperiment.read_attr_hardware


    #--------------------------------------------------------------------------
    #    MetaExperiment command methods
    #--------------------------------------------------------------------------
    
    def dev_state(self):
        """ This command gets the device state (stored in its device_state data member) and returns it to the caller.
        :return: Device state
        :rtype: tango.CmdArgType.DevState
        """
        self.debug_stream("In dev_state()")
        argout = tango.DevState.UNKNOWN
        #----- PROTECTED REGION ID(MetaExperiment.State) ENABLED START -----#
        self.Connect()
        #----- PROTECTED REGION END -----#	//	MetaExperiment.State
        if argout != tango.DevState.ALARM:
            tango.LatestDeviceImpl.dev_state(self)
        return self.get_state()
        
    def dev_status(self):
        """ This command gets the device status (stored in its device_status data member) and returns it to the caller.
        :return: Device status
        :rtype: str
        """
        self.debug_stream("In dev_status()")
        #----- PROTECTED REGION ID(MetadataManager.Status) ENABLED START -----#
        self.Connect()
        #----- PROTECTED REGION END -----#    //    MetadataManager.Status
        self.__status = tango.LatestDeviceImpl.dev_status(self)
        return self.__status

    #----- PROTECTED REGION ID(MetaExperiment.programmer_methods) ENABLED START -----#
    def Connect(self):
        self.debug_stream("In Connect()")
        try:
            beat = self.get_attribute_poll_period('State') * 1.2
        except Exception as e:
            self.warn_stream("Error retrieving poll value: %s" %e)
            beat = 0
        self.debug_stream("Setting heartbeat to %d" %beat)
        self.client.setHeartBeat(beat)
        self.client.beat()
        self.debug_stream("Connection status: %s" %self.client.getState())
        self.UpdateState()

    def CreateInvestigation(self, name):
        self.info_stream("Creating new investigation: " + name)
        inv = investigation(name, self.beamlineID.lower(), datetime.now())
        self.client.sendObject(inv)

    def UpdateState(self):
        if not self.attr_proposal_read or (self.attr_proposal_read == "please enter"):
            self.set_status("No experiment started")
        elif self.attr_sample_read and (self.attr_sample_read != "please enter"):
            self.set_status("Ready to start dataset for experiment " + self.attr_proposal_read)
        else:
            self.set_status("Experiment " + self.attr_proposal_read + " started")
        try:
            self.append_status(self.client.getStatus(),True)
        except Exception:
            self.debug_stream("Error appending status")

    #----- PROTECTED REGION END -----#	//	MetaExperiment.programmer_methods

class MetaExperimentClass(tango.DeviceClass):
    #--------- Add you global class variables here --------------------------
    #----- PROTECTED REGION ID(MetaExperiment.global_class_variables) ENABLED START -----#
    
    #----- PROTECTED REGION END -----#	//	MetaExperiment.global_class_variables


    #    Class Properties
    class_property_list = {
        'graylogServer':
            [tango.DevString, 
            "Graylog Server",
            ["graylog-dau.esrf.fr"] ],
        'graylogPort':
            [tango.DevInt, 
            "Graylog port",
            [12203] ],
        'queueURLs':
            [tango.DevVarStringArray, 
            "URLs of the queues (including port)",
            ["bcu-mq-01.esrf.fr:61613"]],
        'queueName':
            [tango.DevString, 
            "Name of the stomp queue for sending ingestion requests",
            ["/queue/icatIngest"] ],
        }


    #    Device Properties
    device_property_list = {
        'beamlineID':
            [tango.DevString, 
            "Beamline ID for reporting metadata",
            [] ],
        }


    #    Command definitions
    cmd_list = {
        }


    #    Attribute definitions
    attr_list = {
        'proposal':
            [[tango.DevString,
            tango.SCALAR,
            tango.READ_WRITE],
            {
                'Memorized':"true_without_hard_applied"
            } ],
        'sample':
            [[tango.DevString,
            tango.SCALAR,
            tango.READ_WRITE],
            {
                'Memorized':"true_without_hard_applied"
            } ],
        'dataRoot':
            [[tango.DevString,
            tango.SCALAR,
            tango.READ_WRITE],
            {
                'Display level': tango.DispLevel.EXPERT,
                'Memorized':"true_without_hard_applied"
            } ],
        }


def main():
    try:
        py = tango.Util(sys.argv)
        py.add_class(MetaExperimentClass, MetaExperiment, 'MetaExperiment')
        #----- PROTECTED REGION ID(MetaExperiment.add_classes) ENABLED START -----#
        
        #----- PROTECTED REGION END -----#	//	MetaExperiment.add_classes

        U = tango.Util.instance()
        U.server_init()
        U.server_run()

    except tango.DevFailed as e:
        print ('-------> Received a DevFailed exception:',e)
    except Exception as e:
        print ('-------> An unforeseen exception occured....',e)

if __name__ == '__main__':
    main()
